﻿using IceCreamShopBusinessLogic.BusinessLogic;
using IceCreamShopContracts.BindingModels;
using IceCreamShopContracts.BusinessLogicsContracts;
using IceCreamShopDataModels;
using IceCreamShopDataModels.Enums;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace IceCreamShop
{
    public partial class MainForm : Form
    {
        private readonly ILogger _logger;
        private readonly IOrderLogic _orderLogic;
        private readonly IReportLogic _reportLogic;
        public MainForm(ILogger<MainForm> logger, IOrderLogic orderLogic, IReportLogic reportLogic)
        {
            InitializeComponent();
            _logger = logger;
            _orderLogic = orderLogic;
            _reportLogic = reportLogic;
        }

        private void КомпонентыStripMenuItem_Click(object sender, EventArgs e)
        {
            var service =
Program.ServiceProvider?.GetService(typeof(ComponentsForm));
            if (service is ComponentsForm form)
            {
                form.ShowDialog();
            }

        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            LoadData();
        }

        private void LoadData()
        {
            _logger.LogInformation("Загрузка заказов");
            try
            {
                var list = _orderLogic.ReadList(null);
                if (list != null)
                {
                    DataGridView.DataSource = list;
                    DataGridView.Columns["IceCreamId"].Visible = false;
                    DataGridView.Columns["IceCreamName"].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
                }
                _logger.LogInformation("Загрузка заказов");
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки заказов");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void МороженноеStripMenuItem_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(IceCreamsForm));
            if (service is IceCreamsForm form)
            {
                form.ShowDialog();
            }
        }

        private void CreateOrderButton_Click(object sender, EventArgs e)
        {
            var service =
Program.ServiceProvider?.GetService(typeof(OrderForm));
            if (service is OrderForm form)
            {
                form.ShowDialog();
                LoadData();
            }
        }

        private OrderBindingModel CreateBindingModel(int id, bool isDone = false)
        {
            return new OrderBindingModel
            {
                Id = id,
                IceCreamId = Convert.ToInt32(DataGridView.SelectedRows[0].Cells["IceCreamId"].Value),
                Status = Enum.Parse<OrderStatus>(DataGridView.SelectedRows[0].Cells["Status"].Value.ToString()),
                Count = Convert.ToInt32(DataGridView.SelectedRows[0].Cells["Count"].Value),
                Sum = double.Parse(DataGridView.SelectedRows[0].Cells["Sum"].Value.ToString()),
                DateCreate = DateTime.Parse(DataGridView.SelectedRows[0].Cells["DateCreate"].Value.ToString()),
            };
        }

        private void TakeInWorkButton_Click(object sender, EventArgs e)
        {
            if (DataGridView.SelectedRows.Count == 1)
            {
                int id =
               Convert.ToInt32(DataGridView.SelectedRows[0].Cells["Id"].Value);
                _logger.LogInformation("Заказ №{id}. Меняется статус на 'В работе'", id);
                try
                {
                    var operationResult = _orderLogic.TakeOrderInWork(CreateBindingModel(id));
                    if (!operationResult)
                    {
                        throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
                    }
                    LoadData();
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка передачи заказа в работу");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                }
            }

        }

        private void ReadyButton_Click(object sender, EventArgs e)
        {
            if (DataGridView.SelectedRows.Count == 1)
            {
                int id =
               Convert.ToInt32(DataGridView.SelectedRows[0].Cells["Id"].Value);
                _logger.LogInformation("Заказ №{id}. Меняется статус на 'Готов'",
               id);
                try
                {
                    var operationResult = _orderLogic.FinishOrder(CreateBindingModel(id));
                    if (!operationResult)
                    {
                        throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
                    }
                    LoadData();
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка отметки о готовности заказа");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
    MessageBoxIcon.Error);
                }
            }
        }

        private void IssuedButton_Click(object sender, EventArgs e)
        {
            if (DataGridView.SelectedRows.Count == 1)
            {
                int id =
               Convert.ToInt32(DataGridView.SelectedRows[0].Cells["Id"].Value);
                _logger.LogInformation("Заказ №{id}. Меняется статус на 'Выдан'",
               id);
                try
                {
                    var operationResult = _orderLogic.DeliveryOrder(CreateBindingModel(id));
                    if (!operationResult)
                    {
                        throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
                    }
                    _logger.LogInformation("Заказ №{id} выдан", id);
                    LoadData();
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка отметки о выдачи заказа");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
                   MessageBoxIcon.Error);
                }
            }

        }

        private void RefreshButton_Click(object sender, EventArgs e)
        {
            LoadData();
        }

        private void магазиныToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(ShopsForm));
            if (service is ShopsForm form)
            {
                form.ShowDialog();
            }
        }

        private void поставкиToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(SupplyForm));
            if (service is SupplyForm form)
            {
                form.ShowDialog();
            }
        }

        private void продажиToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service = Program.ServiceProvider?.GetService(typeof(SellForm));
            if (service is SellForm form)
            {
                form.ShowDialog();
            }
        }

        private void списокМороженногоToolStripMenuItem_Click(object sender, EventArgs e)
        {
            using var dialog = new SaveFileDialog { Filter = "docx|*.docx" };
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                _reportLogic.SaveIceCreamToWordFile(new ReportBindingModel
                {
                    FileName = dialog.FileName
                });
                MessageBox.Show("Выполнено", "Успех", MessageBoxButtons.OK,
               MessageBoxIcon.Information);
            }

        }

        private void компонентыПоМороженнымToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service =
Program.ServiceProvider?.GetService(typeof(ReportIceCreamComponentForm));
            if (service is ReportIceCreamComponentForm form)
            {
                form.ShowDialog();
            }

        }

        private void списокЗаказовToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service =
Program.ServiceProvider?.GetService(typeof(ReportOrdersForm));
            if (service is ReportOrdersForm form)
            {
                form.ShowDialog();
            }

        }

        private void списокМагазиновToolStripMenuItem_Click(object sender, EventArgs e)
        {
            using var dialog = new SaveFileDialog { Filter = "docx|*.docx" };
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                _reportLogic.SaveShopsToWordFile(new ReportBindingModel
                {
                    FileName = dialog.FileName
                });
                MessageBox.Show("Выполнено", "Успех", MessageBoxButtons.OK,
               MessageBoxIcon.Information);
            }
        }

        private void мороженноеПоМагазинамToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service =
Program.ServiceProvider?.GetService(typeof(ReportShopsIceCreamsForm));
            if (service is ReportShopsIceCreamsForm form)
            {
                form.ShowDialog();
            }
        }

        private void заказыПоДатамToolStripMenuItem_Click(object sender, EventArgs e)
        {
            var service =
Program.ServiceProvider?.GetService(typeof(ReportDateOrdersForm));
            if (service is ReportDateOrdersForm form)
            {
                form.ShowDialog();
            }
        }
    }
}
