using IceCreamShopBusinessLogic.BusinessLogic;
using IceCreamShopBusinessLogic.OfficePackage.Implements;
using IceCreamShopBusinessLogic.OfficePackage;
using IceCreamShopContracts.BusinessLogicsContracts;
using IceCreamShopContracts.StoragesContracts;
using IceCreamShopDatabaseImplement.Implements;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using NLog.Extensions.Logging;

namespace IceCreamShop
{
    internal static class Program
    {
        private static ServiceProvider? _serviceProvider;
        public static ServiceProvider? ServiceProvider => _serviceProvider;
        /// <summary>
        ///  The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            // To customize application configuration such as set high DPI settings or default font,
            // see https://aka.ms/applicationconfiguration.
            ApplicationConfiguration.Initialize();
            var services = new ServiceCollection();
            ConfigureServices(services);
            _serviceProvider = services.BuildServiceProvider();
            Application.Run(_serviceProvider.GetRequiredService<MainForm>());
        }
        private static void ConfigureServices(ServiceCollection services)
        {
            services.AddLogging(option =>
            {
                option.SetMinimumLevel(LogLevel.Information);
                option.AddNLog("nlog.config");
            });
            services.AddTransient<IComponentStorage, ComponentStorage>();
            services.AddTransient<IShopLogic, ShopLogic>();
            services.AddTransient<IShopStorage, ShopStorage>();
            services.AddTransient<IOrderStorage, OrderStorage>();
            services.AddTransient<IIceCreamStorage, IceCreamStorage>();
            services.AddTransient<IComponentLogic, ComponentLogic>();
            services.AddTransient<IOrderLogic, OrderLogic>();
            services.AddTransient<IReportLogic, ReportLogic>();
            services.AddTransient<IIceCreamLogic, IceCreamLogic>();
            services.AddTransient<MainForm>();
            services.AddTransient<ComponentForm>();
            services.AddTransient<ComponentsForm>();
            services.AddTransient<OrderForm>();
            services.AddTransient<IceCreamForm>();
            services.AddTransient<IceCreamComponentForm>();
            services.AddTransient<IceCreamsForm>();
            services.AddTransient<ShopForm>();
            services.AddTransient<ShopsForm>();
            services.AddTransient<SupplyForm>();
            services.AddTransient<SellForm>();
            services.AddTransient<ReportShopsIceCreamsForm>();
            services.AddTransient<ReportDateOrdersForm>();
            services.AddTransient<ReportIceCreamComponentForm>();
            services.AddTransient<ReportOrdersForm>();
            services.AddTransient<AbstractSaveToExcel, SaveToExcel>();
            services.AddTransient<AbstractSaveToWord, SaveToWord>();
            services.AddTransient<AbstractSaveToPdf, SaveToPdf>();
        }
    }
}
