﻿using IceCreamShopBusinessLogic.BusinessLogic;
using IceCreamShopContracts.BusinessLogicsContracts;
using IceCreamShopContracts.SearchModels;
using IceCreamShopContracts.ViewModels;
using IceCreamShopDataModels.Models;
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace IceCreamShop
{
    public partial class SupplyForm : Form
    {
        private readonly List<IceCreamViewModel>? _iceCreamList;
        private readonly List<ShopViewModel>? _shopsList;
        IShopLogic _shopLogic;
        IIceCreamLogic _iceCreamLogic;
        public int ShopId
        {
            get
            {
                return
       Convert.ToInt32(ShopComboBox.SelectedValue);
            }
            set
            {
                ShopComboBox.SelectedValue = value;
            }
        }
        public int IceCreamId
        {
            get
            {
                return
       Convert.ToInt32(IceCreamComboBox.SelectedValue);
            }
            set
            {
                IceCreamComboBox.SelectedValue = value;
            }
        }

        public IIceCreamModel? IceCreamModel
        {
            get
            {
                if (_iceCreamList == null)
                {
                    return null;
                }
                foreach (var elem in _iceCreamList)
                {
                    if (elem.Id == IceCreamId)
                    {
                        return elem;
                    }
                }
                return null;
            }
        }


        public int Count
        {
            get { return Convert.ToInt32(CountTextBox.Text); }
            set
            { CountTextBox.Text = value.ToString(); }
        }
        public SupplyForm(IIceCreamLogic iceCreamLogic, IShopLogic shopLogic)
        {
            InitializeComponent();
            _shopLogic = shopLogic;
            _iceCreamLogic = iceCreamLogic;
            _iceCreamList = iceCreamLogic.ReadList(null);
            _shopsList = shopLogic.ReadList(null);
            if (_iceCreamList != null)
            {
                IceCreamComboBox.DisplayMember = "IceCreamName";
                IceCreamComboBox.ValueMember = "Id";
                IceCreamComboBox.DataSource = _iceCreamList;
                IceCreamComboBox.SelectedItem = null;
            }
            if (_shopsList != null)
            {
                ShopComboBox.DisplayMember = "ShopName";
                ShopComboBox.ValueMember = "Id";
                ShopComboBox.DataSource = _shopsList;
                ShopComboBox.SelectedItem = null;
            }
        }

        private void SaveButton_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(CountTextBox.Text))
            {
                MessageBox.Show("Заполните поле Количество", "Ошибка",
               MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (IceCreamComboBox.SelectedValue == null)
            {
                MessageBox.Show("Выберите мороженное", "Ошибка",
               MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (ShopComboBox.SelectedValue == null)
            {
                MessageBox.Show("Выберите магазин", "Ошибка",
               MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            try
            {
                int count = Convert.ToInt32(CountTextBox.Text);

                bool res = _shopLogic.MakeSupply(
                    new ShopSearchModel() { Id = Convert.ToInt32(ShopComboBox.SelectedValue) },
                    _iceCreamLogic.ReadElement(new() { Id = Convert.ToInt32(IceCreamComboBox.SelectedValue) }),
                    count
                );

                if (!res)
                {
                    throw new Exception("Ошибка при пополнении. Дополнительная информация в логах");
                }

                MessageBox.Show("Пополнение прошло успешно");
                DialogResult = DialogResult.OK;
                Close();

            }
            catch (Exception err)
            {
                MessageBox.Show("Ошибка пополнения");
                return;
            }
        }

        private void CancelButton_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
