﻿using IceCreamShopContracts.BindingModels;
using IceCreamShopContracts.SearchModels;
using IceCreamShopContracts.StoragesContracts;
using IceCreamShopContracts.ViewModels;
using IceCreamShopDataModels.Models;
using IceCreamShopFileImplement;
using IceCreamShopFileImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace IceCreamShopFileImplement.Implements
{
    public class ShopStorage : IShopStorage
    {
        private readonly DataFileSingleton _source;
        public ShopStorage()
        {
            _source = DataFileSingleton.GetInstance();
        }
        public List<ShopViewModel> GetFullList()
        {
            return _source.Shops
 .Select(x => x.GetViewModel)
.ToList();
        }
        public List<ShopViewModel> GetFilteredList(ShopSearchModel
       model)
        {
            if (string.IsNullOrEmpty(model.Name))
            {
                return new();
            }
            return _source.Shops
 .Where(x => x.ShopName.Contains(model.Name))
.Select(x => x.GetViewModel)
.ToList(); ;
        }
        public ShopViewModel? GetElement(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.Name) && !model.Id.HasValue)
            {
                return null;
            }
            return _source.Shops
 .FirstOrDefault(x =>
(!string.IsNullOrEmpty(model.Name) && x.ShopName ==
model.Name) ||
 (model.Id.HasValue && x.Id == model.Id))
 ?.GetViewModel;
        }
        public ShopViewModel? Insert(ShopBindingModel model)
        {
            model.Id = _source.Shops.Count > 0 ? _source.Shops.Max(x =>
x.Id) + 1 : 1;
            var newShop = Shop.Create(model);
            if (newShop == null)
            {
                return null;
            }
            _source.Shops.Add(newShop);
            _source.SaveShops();
            return newShop.GetViewModel;
        }
        public ShopViewModel? Update(ShopBindingModel model)
        {
            var component = _source.Shops.FirstOrDefault(x => x.Id ==
model.Id);
            if (component == null)
            {
                return null;
            }
            component.Update(model);
            _source.SaveShops();
            return component.GetViewModel;
        }
        public ShopViewModel? Delete(ShopBindingModel model)
        {
            var element = _source.Shops.FirstOrDefault(x => x.Id ==
           model.Id);
            if (element != null)
            {
                _source.Shops.Remove(element);
                _source.SaveShops();
                return element.GetViewModel;
            }
            return null;
        }

        public bool CheckAvailability(int iceCreamId, int count)
        {
            int minus = _source.Shops.Select(x => x.ShopIceCreams.Select(y => (y.Value.Item1.Id == iceCreamId ? y.Value.Item2 : 0)).Sum()).Sum();
            count -= minus;
            return count <= 0;
        }

        public bool SellIceCreams(IIceCreamModel model, int count)
        {
            var iceCream = _source.IceCreams.FirstOrDefault(x => x.Id == model.Id);

            if (iceCream == null || !CheckAvailability(iceCream.Id, count))
            {
                return false;
            }

            for (int i = 0; i < _source.Shops.Count; i++)
            {
                var shop = _source.Shops[i];
                var icecreams = shop.ShopIceCreams;

                foreach (var icecream in icecreams.Where(x => x.Value.Item1.Id == iceCream.Id))
                {
                    var min = Math.Min(icecream.Value.Item2, count);
                    icecreams[icecream.Value.Item1.Id] = (icecream.Value.Item1, icecream.Value.Item2 - min);
                    count -= min;

                    if (count <= 0)
                    {
                        break;
                    }
                }

                shop.Update(new ShopBindingModel
                {
                    Id = shop.Id,
                    ShopName = shop.ShopName,
                    Address = shop.Address,
                    DateOpen = shop.DateOpen,
                    MaxCapacity = shop.MaxCapacity,
                    ShopIceCreams = icecreams
                });
            }
            _source.SaveShops();
            return true;
        }

    }
}
