﻿using IceCreamShopContracts.BindingModels;
using IceCreamShopContracts.ViewModels;
using IceCreamShopDataModels.Models;
using IceCreamShopFileImplement;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Threading.Tasks.Dataflow;
using System.Xml.Linq;

namespace IceCreamShopFileImplement.Models
{
    public class Shop : IShopModel
    {
        public int Id { get; private set; }
        public string ShopName { get; private set; }
        public string Address { get; private set; }
        public DateTime DateOpen { get; private set; }
        public Dictionary<int, int> IceCreams { get; private set; } = new();
        private Dictionary<int, (IIceCreamModel, int)>? _shopIceCreams =
null;
        public Dictionary<int, (IIceCreamModel, int)> ShopIceCreams
        {
            get
            {
                if (_shopIceCreams == null)
                {
                    var source = DataFileSingleton.GetInstance();
                    _shopIceCreams = IceCreams.ToDictionary(x => x.Key, y =>
                    ((source.IceCreams.FirstOrDefault(z => z.Id == y.Key) as IIceCreamModel)!,
                    y.Value));
                }
                return _shopIceCreams;
            }
        }

        public int MaxCapacity { get; private set; }

        public static Shop? Create(ShopBindingModel model)
        {
            if (model == null)
                return null;
            return new Shop()
            {
                Id = model.Id,
                ShopName = model.ShopName,
                Address = model.Address,
                DateOpen = model.DateOpen,
                MaxCapacity = model.MaxCapacity,
                IceCreams = model.ShopIceCreams.ToDictionary(x => x.Key, x => x.Value.Item2)
            };
        }

        public static Shop? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            return new Shop()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                ShopName = element.Element("ShopName")!.Value,
                Address = element.Element("Address")!.Value,
                MaxCapacity = Convert.ToInt32(element.Element("MaxCapacity")!.Value),
                DateOpen = Convert.ToDateTime(element.Element("DateOpen")!.Value),
                IceCreams =
           element.Element("ShopIceCreams")!.Elements("ShopIceCream")
            .ToDictionary(x =>
           Convert.ToInt32(x.Element("Key")?.Value), x =>
           Convert.ToInt32(x.Element("Value")?.Value))
            };
        }


        public void Update(ShopBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            ShopName = model.ShopName;
            Address = model.Address;
            DateOpen = model.DateOpen;
            MaxCapacity = model.MaxCapacity;
            if (model.ShopIceCreams.Count > 0)
            {
                IceCreams = model.ShopIceCreams.ToDictionary(x => x.Key, x => x.Value.Item2);
                _shopIceCreams = null;
            }
        }

        public ShopViewModel GetViewModel => new()
        {
            Id = Id,
            ShopName = ShopName,
            Address = Address,
            DateOpen = DateOpen,
            MaxCapacity = MaxCapacity,
            ShopIceCreams = ShopIceCreams
        };

        public XElement GetXElement => new("Shop",
             new XAttribute("Id", Id),
             new XElement("ShopName", ShopName),
             new XElement("Address", Address),
             new XElement("DateOpen", DateOpen),
             new XElement("MaxCapacity", MaxCapacity),
             new XElement("ShopIceCreams", IceCreams
                 .Select(x => new XElement("ShopIceCream",
                     new XElement("Key", x.Key),
                     new XElement("Value", x.Value))
                 ).ToArray()));
    }
}
